class Base:
    name = ''
    symbol = ''
    factor = 1


class Yotta( Base ):
    name = 'yotta'
    symbol = 'Y'
    factor = 10 ** 24


class Zetta( Base ):
    name = 'zetta'
    symbol = 'Z'
    factor = 10 ** 21


class Exa( Base ):
    name = 'exa'
    symbol = 'E'
    factor = 10 ** 18


class Peta( Base ):
    name = 'peta'
    symbol = 'P'
    factor = 10 ** 15


class Tera( Base ):
    name = 'tera'
    symbol = 'T'
    factor = 10 ** 12


class Giga( Base ):
    name = 'giga'
    symbol = 'G'
    factor = 10 ** 9


class Mega( Base ):
    name = 'mega'
    symbol = 'M'
    factor = 10 ** 6


class Kilo( Base ):
    name = 'kilo'
    symbol = 'k'
    factor = 10 ** 3


class Hecto( Base ):
    name = 'hecto'
    symbol = 'h'
    factor = 10 ** 2


class Deca( Base ):
    name = 'deca'
    symbol = 'da'
    factor = 10 ** 1


class Centi( Base ):
    name = 'Centi'
    symbol = 'c'
    factor = 10 ** -2


class Mili( Base ):
    name = 'mili'
    symbol = 'm'
    factor = 10 ** -3


class Micro( Base ):
    name = 'micro'
    symbol = 'μ'
    factor = 10 ** -6


class Nano( Base ):
    name = 'nano'
    symbol = 'n'
    factor = 10 ** -9


class Pico( Base ):
    name = 'pico'
    symbol = 'p'
    factor = 10 ** -12


class Femto( Base ):
    name = 'femto'
    symbol = 'f'
    factor = 10 ** -15


class Atto( Base ):
    name = 'Atto'
    symbol = 'a'
    factor = 10 ** -18


class Zepto( Base ):
    name = 'zepto'
    symbol = 'z'
    factor = 10 ** -21


class Yocto( Base ):
    name = 'yocto'
    symbol = 'y'
    factor = 10 ** -24
